@@ -991,7 +991,7 @@ prepare_error_tgs (struct kdc_request_state *state,
 {
     krb5_error errpkt;
     krb5_error_code retval = 0;
-    krb5_data *scratch;
+    krb5_data *scratch, *fast_edata = NULL;
 
     errpkt.ctime = request->nonce;
     errpkt.cusec = 0;
@@ -1014,15 +1014,20 @@ prepare_error_tgs (struct kdc_request_state *state,
         return ENOMEM;
     }
     errpkt.e_data = *e_data;
-    if (state)
-        retval = kdc_fast_handle_error(kdc_context, state, request, NULL, &errpkt);
+    if (state) {
+        retval = kdc_fast_handle_error(kdc_context, state, request, NULL,
+                                       &errpkt, &fast_edata);
+    }
     if (retval) {
         free(scratch);
         free(errpkt.text.data);
         return retval;
     }
+    if (fast_edata)
+        errpkt.e_data = *fast_edata;
     retval = krb5_mk_error(kdc_context, &errpkt, scratch);
     free(errpkt.text.data);
+    krb5_free_data(kdc_context, fast_edata);
     if (retval)
         free(scratch);
     else
